/*******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software
 * and to discontinue the availability of this software. By using this software,
 * you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer*
 * Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
 *******************************************************************************/
/**************************************************************************//**
* @file         main.c
* $Rev: 879 $
* $Date:: 2013-07-03 09:29:09 +0900#$
* @brief        BSP main task
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include "cmsis_os.h"

#include "Serial.h"

#include "bsp_util.h"

#include "sample_main.h"
/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
/*----------------------------------------------------------------------------
 *   Debug UART sample Global:
 *---------------------------------------------------------------------------*/

void debug_uart_task(void const *);
osThreadId debug_uart_task_id;
osThreadDef(debug_uart_task, osPriorityNormal, 1, 0);
void led1_blink( void const * arg );
void led2_blink( void const * arg );
osThreadDef(led1_blink, osPriorityNormal, 1, 0);
osTimerDef(led2, led2_blink);


/**************************************************************************//**
* Function Name: main
* @brief         CMSIS RTOS Task : Main task
*
*                Description:<br>
*                Application initialize task
* @param         none
* @retval        none
******************************************************************************/
 void main(void)
{
    /***********************************************************************/
    /* Initialise Debug UART */
    /***********************************************************************/

    SER_Init();

    /***********************************************************************/
    /* Print Board Information */
    /***********************************************************************/

    R_BSP_PrintBootMode();
/*
    RTX_AZ_Init(LogBuffer, 0x10000);
    RTX_AZ_Start();
*/

    /* ==== Ӌ@\̃NbNJn ==== */
    STB_Init();

    /* ==== PORT setting ==== */
    PORT_Init();

    /* ==== Initialize Terminal ==== */
    /* SCIF 3ch */
    /* P1=66.67MHz CKS=0 SCBRR=17 Bit rate error=0.46% => Baud rate=115200bps */
    IoInitScif3();

	/* ^C}`(JԂ^C},Ȃ) */
	osTimerId led2_id = osTimerCreate(osTimer(led2), osTimerPeriodic, NULL);

	/* LED(P7.9)[qݒ(0o,p@\gp) */
	GPIOPSR7=0x02000000;
	GPIOPMSR7=0x02000000;
	GPIOPMCSR7=0x02000000;

	/* ^C}300msœJn */
	osTimerStart(led2_id, 300);
	osThreadId blink1_id = osThreadCreate(osThread(led1_blink),NULL);
    while(1)
	{
		osDelay(1);
	}
}

void led1_blink( void const * arg )
{
	/* LED(P7.8)[qݒ(0o,p@\gp) */
	GPIOPSR7=0x01000000;
	GPIOPMSR7=0x01000000;
	GPIOPMCSR7=0x01000000;

	while(1)
	{
		/* 500ms҂ */
		osDelay( 500 );

		/* LED(P7.8)] */
		if ((GPIOP7 & 0x0100) == 0)
		{
			GPIOPSR7=0x01000100;
		}
		else
	{
			GPIOPSR7=0x01000000;
		}
	}
}

void led2_blink( void const * arg )
{
	/* LED(P7.9)] */
	if ((GPIOP7 & 0x0200) == 0)
	{
		GPIOPSR7=0x02000200;
	}
	else
	{
		GPIOPSR7=0x02000000;
	}
}

/**************************************************************************//**
* Function Name: debug_uart_task
* @brief         CMSIS RTOS Task : Debug UART task
*
*                Description:<br>
*                Debug UART sample
* @param[in]     unused: unuse
* @retval        none
******************************************************************************/
void debug_uart_task(void const * unused)
{
    while (1)
    {
        osDelay (1500);
        SER_PutChar('b');
    }

    //osThreadTerminate(osThreadGetId());
}


/*----------------------------------------------------------------------------
 * end of file
 *---------------------------------------------------------------------------*/
